@@ -8,8 +8,8 @@
 # Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
 # 2006, 2007, 2008, 2009, 2010, 2011 Comite Reseau des Universites
 # Copyright (c) 2011, 2012, 2013, 2014, 2015, 2016, 2017 GIP RENATER
-# Copyright 2017, 2018 The Sympa Community. See the AUTHORS.md file at the
-# top-level directory of this distribution and at
+# Copyright 2017, 2018, 2020 The Sympa Community. See the AUTHORS.md
+# file at the top-level directory of this distribution and at
 # <https://github.com/sympa-community/sympa.git>.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -204,6 +204,17 @@ sub wrap {
     };
 }
 
+sub _mailbox {
+    my ($context, $email, $comment) = @_;
+
+    return sub {
+        my $text = shift;
+
+        return Sympa::Tools::Text::addrencode($email, $text,
+            Conf::lang2charset($language->get_lang), $comment);
+    };
+}
+
 sub _mailto {
     my ($context, $email, $query, $nodecode) = @_;
 
@@ -297,6 +308,8 @@ sub _get_option_description {
         my $robot_id;
         if (ref $that eq 'Sympa::List') {
             $robot_id = $that->{'domain'};
+        } elsif (ref $that eq 'Sympa::Family') {
+            $robot_id = $that->{'domain'};
         } elsif ($that and $that ne '*') {
             $robot_id = $that;
         } else {
@@ -318,6 +331,7 @@ sub _get_option_description {
             'reception'  => \%Sympa::ListOpt::reception_mode,
             'visibility' => \%Sympa::ListOpt::visibility_mode,
             'status'     => \%Sympa::ListOpt::list_status,
+            'status:cap' => \%Sympa::ListOpt::list_status_capital,
         }->{$type}
             || \%Sympa::ListOpt::list_option;
         my $t = $map->{$option} || {};
@@ -356,7 +370,8 @@ sub _url_func {
 
     my $that = $self->{context};
     my $robot_id =
-          (ref $that eq 'Sympa::List') ? $that->{'domain'}
+          (ref $that eq 'Sympa::List')   ? $that->{'domain'}
+        : (ref $that eq 'Sympa::Family') ? $that->{'domain'}
         : ($that and $that ne '*') ? $that
         :                            '*';
 
@@ -381,9 +396,6 @@ sub parse {
     my %options    = @_;
 
     my @include_path;
-    if ($self->{plugins}) {
-        push @include_path, @{$self->{plugins}->tt2Paths || []};
-    }
     if (defined $self->{context}) {
         push @include_path,
             @{Sympa::get_search_path($self->{context}, %$self) || []};
@@ -406,6 +418,7 @@ sub parse {
             helploc  => [\&maketext, 1],
             locdt    => [\&locdatetime, 1],
             wrap      => [\&wrap,       1],
+            mailbox   => [\&_mailbox,   1],
             mailto    => [\&_mailto,    1],
             mailtourl => [\&_mailtourl, 1],
             obfuscate => [\&_obfuscate, 1],
@@ -558,10 +571,6 @@ Reference to array containing additional template search paths.
 I<Read only>.
 Error occurred at the last execution of parse, or C<undef>.
 
-=item {plugins}
-
-TBD.
-
 =item {subdir}, {lang}, {lang_only}
 
 TBD.
@@ -638,6 +647,29 @@ A string representing date/time:
 
 =back
 
+=item mailbox ( email, [ comment ] )
+
+Generates mailbox string appropriately encoded to suit for addresses
+in header fields.
+
+=over
+
+=item Filtered text
+
+Display name, if any.
+
+=item email
+
+E-mail address.
+
+=item comment
+
+Comment, if any.
+
+=back
+
+This filter was introduced on Sympa 6.2.42.
+
 =item mailto ( email, [ {key =E<gt> val, ...}, [ nodecode ] ] )
 
 Generates HTML fragment linking to C<mailto:> URL,
